// In this program, we will braodcast the value of a variable from the master
// node to all other nodes.
// This is an example of the BCAST MPI function.

#include <math.h>
#include <iostream>
#include "mpi.h"
#include <cstdlib>
using namespace std;


int main(int argc, char *argv[])
{
	int n, rank, size, i;
	int value = 0;
	
	// MPI:Status variable status is a class, and can be used for debugging purposes if need be
	MPI::Status status;

	MPI::Init(argc, argv);
	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();

	// we are going to let the master nodes variable "value" have value of 9, the rest are all 0
	if (rank ==0)
	{
		value = 9;
	}

	// Print out the current value of value, which should be 0 before the broadcast, except for master node
	cout << "Node " << rank << " of " << size << ": Pre broadcast value is " << value << endl;

	// New we have the master node broadcast its "value" to the rest.
	// This is pretty straight forward, address of the variable being sent, how manty, what type, and who is sending
	MPI::COMM_WORLD.Bcast(&value, 1, MPI::INT, 0);

	// Check to make sure that "value" has now been updated
        cout << "Node " << rank << " of " << size << ": Post broadcast value is " << value << endl;

	MPI::Finalize();
	return 0;
}
