#include <math.h>
#include <iostream>
#include "mpi.h"
#include <cstdlib>
using namespace std;


int main(int argc, char *argv[])
{
	int thenumber = 0; // used to test the group BCAST stuff

	int rank, size; // for COMM_WORLD info
	int namelength = 0; // used for printing out the name of the compute node
	int computenodes = 4; // number of compute nodes on cluster, hopefully this will increase


	int group_num = 0; // used to determine which group each node should belong to
	int group_rank = 0; // for the rank within the group
	int group_size = 0; // for the size within the group

	MPI::Init(argc, argv);
	char name[MPI_MAX_PROCESSOR_NAME];  // create character array for the names of the compute nodes

	MPI::Intracomm COMM_GROUP;  // used to create subgroups corresponding to each compute node

	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();
	
	// we can use this for debugging, each process can now tell us what compute node they are on
	MPI::Get_processor_name(name, namelength);

	// this should group all the process on each compute node into the same group_num, assuming even distribution
	group_num = floor(rank/(size/computenodes)); 

	// now we use the MPI.Split to break up our processes based on group number and their current rank
	// then note that we get the new group size and group rank almost like we do at MPI initialize time...
	COMM_GROUP = MPI::COMM_WORLD.Split(group_num, rank);
	group_size = COMM_GROUP.Get_size();
	group_rank = COMM_GROUP.Get_rank();

	// let's now test everything, make sure we have our groups built correctly
        cout << "Process " << rank <<"/" << size << "(" << name << "): Group - " << group_num <<  ", Group size - " << group_size << ", Group rank - " << group_rank << endl;

	// now let's play around with local variables
	if (group_rank == 0)
	{
		thenumber = 1000 + group_num;
	}

        cout << "Process " << rank <<"/" << size << "(" << name << "): Group - " << group_num <<  ", Group size - " << group_size << ", Group rank - " << group_rank << ", before Bcast # = " << thenumber << endl;

	// now let's do a local braodcast!
	COMM_GROUP.Bcast(&thenumber, 1, MPI::INT,0);
        cout << "Process " << rank <<"/" << size << "(" << name << "): Group - " << group_num <<  ", Group size - " << group_size << ", Group rank - " << group_rank << ", after Bcast # = " << thenumber << endl;

	// free up memory be deleting the subcommunications group
	COMM_GROUP.Free();

	MPI::Finalize();
	return 0;
}
