// In this program, we will play the guessing game.  Process 0 will pick a random
// number between 1 and N, where N is input at runtime by the user.  Process 0
// will then tell all the other processes what N is, and they will start guessing
// until the correct number has been found.  Guessing will occur in rounds, and
// this will be implemented using the MPI_Gather function.


// Please read the plentiful comments on this program, some of them are rather
// important!

#include <math.h>
#include <iostream>
#include "mpi.h"
#include <cstdlib>
#include <ctime>
using namespace std;


int main(int argc, char *argv[])
{
	int ierr;
	int n, rank, size, i;
	int N  = 0;  // largest value for the guessing game, input from command line
	int value = 0; 

//      the variable "thenumber" will be the number that the master process generates 
//	for the guessing game, and it will be the variable that the compute processes
//	use to try to guess the number
	int thenumber = 0; // the number master node generates for the guessing game

//	N = atoi(argv[1]); // input of largest possible number for guessing game, done command line

//	For some reason, dynamically allocating arrays causes problems with MPI_Gather later on
//	Calling MPI_Gather more than once causes a core dump.  Furthermore, if we dynamically allocate
//	memory for gatherV ONLY on the master node, we end up with compile problems if we try to 
//	reference gatherV later on.
//
//	int *gatherV = new int[size] (); // initialize the array to all zeros

	int gatherV[40];  // cheap hack for now, our cluster has max 40 processes
	
	// MPI:Status variable status is a class, and can be used for debugging purposes if need be
	MPI::Status status;

	MPI::Init(argc, argv);
	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();

	// make sure the first 0---size-1 entries are zeroed out in the gatherV array on  master node 
	if (rank == 0)
	{
		N = atoi(argv[1]); // input of largest possible number for guessing game, done command line

		cout << "Node " << rank << " of " << size << ": Input integer is " << N << endl;
		for (i=0; i<size; i++)
		{
			gatherV[i] = 0;
			cout << "Node " << rank << " of " << size << ": gatherv[" << i << "] = " << gatherV[i] << endl;
		}
		cout << endl;
	
		srand(time(NULL));	
		thenumber = (rand() % N) + 1;
		cout << "Node " << rank << " of " << size << ": My secret number is " << thenumber << endl;
	}

	cout << "Node " << rank << " of " << size << ": Current value of N is  " << N << endl;

//	So the master process will now broadcast the maximum possible number that will have to be guessed	
	MPI::COMM_WORLD.Bcast(&N, 1, MPI::INT, 0);

	cout << "Node " << rank << " of " << size << ": Updated value of N is  " << N << endl;

//	For some reason, I cannot get the fancy C++ version to work, so I am using the C code syntax
//	The following line contains what, in my opinion, should work, but I cannot seem to get it do compile
//	MPI::Intracomm::Gather(&rank, 1, MPI::INT, gatherV, 1, MPI::INT, 0);  

//	So now as a test, we will have each process send their value of rank to process 0, where each
//	entry is stored in the array according to process number, this includes process 0!
	MPI_Gather(&rank, 1, MPI_INT, gatherV, 1, MPI_INT, 0, MPI_COMM_WORLD);

//	We now print out the gathered values which have been stored in the gatherV array
//	and were collected from ALL the nodes, just for verification.
	if (rank == 0)
	{
		for (i=0; i<size; i++)
		{
			cout << "Node " << rank << " of " << size << ": gatherv[" << i << "] = " << gatherV[i] << endl;
		}
		cout << endl;
	}

//	Now, we start the actual guessing game.  Each process which does not have rank 0 will be creating a 
//	guess between 1 and N.  This value will be stored in "thenumber"
	if (rank != 0)
	{

		srand(rank);	
		thenumber = (rand() % N) + 1;
		cout << "Node " << rank << " of " << size << ": I am guessing the number " << thenumber << endl;
	}

//	Not sure if we will need barrier, we shall see once we do a bunch of MPI_Gathers.  This is here 
//	essentially for syntax and as an example.
//	MPI_Barrier(MPI_COMM_WORLD);

//	Ok, now for the first time, we have each process send their guess, stored in "thenumber" to process 0
//	where they will all be stored in the array "gatherV". Note that process 0's number is stored in entry [0].
	MPI_Gather(&thenumber, 1, MPI_INT, gatherV, 1, MPI_INT, 0, MPI_COMM_WORLD);

//	Obviously process 0 should check to see if any entries > 0 in the array match enty 0, but
//	for debugging purposes, all we are doing here is printing out the entries gathered.
	if (rank == 0)
	{
		for (i=0; i<size; i++)
		{
			cout << "Node " << rank << " of " << size << ": gatherv[" << i << "] = " << gatherV[i] << endl;
		}
	}

//	If we were to get the array dynamically allocated, we would need to unallocate with the following command
//	delete [] gatherV;

	MPI::Finalize();
	return 0;
}
