// In this program, we will play the guessing game.  Process 0 will pick a random
// number between 1 and N, where N is input at runtime by the user.  Process 0
// will then tell all the other processes what N is, and they will start guessing
// until the correct number has been found.  Guessing will occur in rounds, and
// this will be implemented using the MPI Gather function.


// Please read the plentiful comments on this program, some of them are rather
// important!

#include <math.h>
#include <iostream>
#include "mpi.h"
#include <cstdlib>
#include <ctime>
using namespace std;


int main(int argc, char *argv[])
{
	int ierr;
	int n, rank, size, i;
	int N  = 0;  // largest value for the guessing game, input from command line
	int value = 0; 
	int computeguess = 0; // variable to store guess on master process from compute process
	int recvcount = 0; // master process can keep track of how many messages they have received

	int gong = 0; //(g)uess (o)r (n)ot (g)uess
	int cntr = 0; // number of guesses

	int thenumber = 0; // the number master node generates for the guessing game

// 	MPI:Status variable status is a class, and can be used to get message info
	MPI::Status status;

	MPI::Init(argc, argv);
	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();

	if (rank == 0)
	{
		N = atoi(argv[1]); // input of largest possible number for guessing game, done command line

		cout << "Node " << rank << " of " << size << ": Input integer is " << N << endl;

//		now it is time to pick the number that the other processes will try to guess	
		srand(time(NULL));	
		thenumber = (rand() % N) + 1;
		cout << "Node " << rank << " of " << size << ": My secret number is " << thenumber << endl;
	}

//	cout << "Node " << rank << " of " << size << ": Current value of N is " << N << endl;

//	So the master process will now broadcast the maximum possible number that will have to be guessed	
	MPI::COMM_WORLD.Bcast(&N, 1, MPI::INT, 0);

	cout << "Node " << rank << " of " << size << ": Received value of N is " << N << endl;

//	Now, we start the actual guessing game.  Each process which does not have rank 0 will be creating a 
//	guess between 1 and N.  This value will be stored in "thenumber"
	if (rank != 0)
	{
		srand((rank+1)*time(NULL));	
		thenumber = (rand() % N) + 1;
//		cout << "Node " << rank << " of " << size << ": I am guessing the number " << thenumber << endl;
	}


//	https://www.mpi-forum.org/docs/mpi-2.1/mpi21-report-bw/node45.htm see here for C++ bindings on Status class
	if (rank == 0)
	{
		while (gong==0)
		{
			MPI::COMM_WORLD.Recv(&computeguess, 1, MPI::INT, MPI::ANY_SOURCE, 0, status);
//			cout << "Node " << rank << " of " << size << ": Receiving guess of " << computeguess << " from process " << status.MPI::Status::Get_source() << endl;
			if(computeguess==thenumber)
			{
				gong=status.MPI::Status::Get_source();
				cout<< "Node " << rank << "of " << size << ": We have a winner! Number of guesses is "<< recvcount << " and winner is: "<<status.MPI::Status::Get_source()<<endl;
			}

			MPI::COMM_WORLD.Send(&gong,1,MPI::INT,status.MPI::Status::Get_source(),0);			

			recvcount++;
		}
		for (int i=0;i<(size-2);i++)
		{
			MPI::COMM_WORLD.Recv(&computeguess, 1, MPI::INT, MPI::ANY_SOURCE, 0, status);
			cout << "Node " << rank << " of " << size << ": Receiving guess of " << computeguess << " from process " << status.MPI::Status::Get_source() << endl;
			MPI::COMM_WORLD.Send(&gong,1,MPI::INT,status.MPI::Status::Get_source(),0);			

		}
	}
	cntr++;
	if (rank !=0)
	{
		while(gong==0)
			{
		MPI::COMM_WORLD.Send(&thenumber, 1, MPI::INT, 0, 0);
		MPI::COMM_WORLD.Recv(&gong, 1, MPI::INT, MPI::ANY_SOURCE, 0, status);
//		cout << "Node " << rank << " of " << size << ": Sending master process my guess of " << thenumber << endl;
		srand((rank+1)*time(NULL)*cntr);	
		thenumber = (rand() % N) + 1;
		cntr++;
		}
		cout << "Node " << rank << " of " << size << ": Terminating program.  Numer of guesses was " << cntr << endl;
	}

	MPI::Finalize();
	return 0;
}
