// In this program, we will take a variable from each node, and have them added together
// and then stored in a variable on the master node (process 0).
// This is an example of the REDUCE MPI function.

#include <math.h>
#include <iostream>
#include "mpi.h"
#include <cstdlib>
using namespace std;


int main(int argc, char *argv[])
{
	int n, rank, size, i;
	int value = 0;
	
	// MPI:Status variable status is a class, and can be used for debugging purposes if need be
	MPI::Status status;

	MPI::Init(argc, argv);
	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();

	// Print out the current value of value, which should be 0 before the broadcast, except for master node
	cout << "Node " << rank << " of " << size << ": Pre reduce value is " << value << endl;

	// the value in rank is sent to node 0, and their sum is stored in the variable value, on master node only
	MPI::COMM_WORLD.Reduce(&rank, &value, 1, MPI::INT, MPI::SUM, 0);

	// Check to make sure that "value" has now been updated on master node, and no other
        cout << "Node " << rank << " of " << size << ": Post broadcast value is " << value << endl;

	MPI::Finalize();
	return 0;
}
