// In this program, we will simply just send a message to the next ranked node (until the last
// node, who will send to node 0).  
// This is an example of the the SEND and RECV MPI functions.

#include <math.h>
#include <iostream>
#include "mpi.h"
#include <cstdlib>
using namespace std;


int main(int argc, char *argv[])
{
	int n, rank, size, i;
	int value = 0;
	
	// MPI:Status variable status is a class, and can be used for debugging purposes if need be
	MPI::Status status;

	MPI::Init(argc, argv);
	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();

	// This is just a check to make sure we are going to send and receive correctly, check that math!
	cout << "Node " << rank << " of " << size << ": Next node in line has rank " << (rank+1+size)%size << endl;
	cout << "Node " << rank << " of " << size << ": Previous node in line has rank " << (rank-1+size)%size << endl;

	// Print out the current value of value, which should be 0 before the messaging
        cout << "Node " << rank << " of " << size << ": Current value is " << value << endl;

	// Now we send the value of "rank" to the node with rank 1 greater, and they will store it in "value"
	// Note the send command is sending "rank", and in the receive command it is stored in "value"
	MPI::COMM_WORLD.Send(&rank, 1, MPI::INT, (rank+1+size)%size, 0);
        MPI::COMM_WORLD.Recv(&value, 1, MPI::INT, (rank-1+size)%size, 0, status);

	// Check to make sure that "value" has now been updated
        cout << "Node " << rank << " of " << size << ": New value is " << value << endl;

	MPI::Finalize();
	return 0;
}
