// This program is a simple demonstration of creating a directory on each compute node and storing 
// data files in the created directory.

// A great example of how to run this program would be by using the computeLocalHD.q job queue
// with the mpirr (mpi round robin) parallel environment (pe), as in the following command:
// qsub -q computeLocalHD.q -pe mpirr 12 subFileFlex.qsub MPIFileOpenTest1

// The directory /state/partition1/CycleData will be created on each compute node that is in the
// queue, and all subsequent data will be stored in that directory, in fact, we create text files
// for each compute process and the rank of each process is stored in the text file they create
// and open.

#include <math.h>
#include <iostream>
#include "mpi.h"
#include <fstream>
#include <cstdlib>
#include <string>
#include <sstream>

using namespace std;


// This is a simple function which appends the rank to the text file name 
string itos(int i)
{
        stringstream s;
        s<<i;
        return s.str();
}


int main(int argc, char *argv[])
{
	int rank, size;

//	The start of the MPI structure, generate rank and size
	MPI::Init(argc, argv);
	size = MPI::COMM_WORLD.Get_size();
	rank = MPI::COMM_WORLD.Get_rank();

//	All the file I/O stuff, create file name, and ofstream pointer
	string filename;
	ofstream mylittlefile;

//	Linux Only -- create the directories on each compute node, if necessary
//	Pay attention to the path!
	system("mkdir -p /state/partition1/CycleData");

//	Open the file, keep this the same path as the directory created above
	filename = "/state/partition1/CycleData/myfile" + itos(rank+1) + ".txt";
	mylittlefile.open(filename.c_str());

//	Put some stuff in that file, include rank so we know how things are ordered by rank
	mylittlefile << "Node " << rank << ": This is a test, this is only a test.  Soon we will have cycle data here!" << endl;

	mylittlefile.close();

	MPI::Finalize();
	return 0;
}
